import time
import httpx
import traceback
import json
from pathlib import Path
from base64 import b64encode,b64decode
from nonebot import get_driver
from nonebot.log import logger
from urllib3 import Timeout
from ..utils import byte2md5

config =  get_driver().config
ocr_url = config.ocr_url
dir_path = Path(__file__).parent.parent
game_path = Path(__file__).parent
ocr_data_path = game_path/ 'ocr_data.json'
upload_url = "https://api.wows.shinoaki.com/api/wows/cache/image/ocr"
download_url = "https://api.wows.shinoaki.com/api/wows/cache/image/ocr"

headers = {
    'Authorization': config.api_token
}

async def pic2txt_byOCR(img_path,filename):
    try:
        global ocr_filename_data
        if filename in ocr_filename_data:
            logger.success(f"filename匹配，跳过OCR:{filename}")
            return b64decode(ocr_filename_data[filename]).decode('utf-8')
        if config.ocr_offline:
            return ''
        async with httpx.AsyncClient() as client:
            #logger.success(f"图片地址{img_path}")
            #resp = await client.get(img_path)
            #img_base64 = str(b64encode(resp.content),encoding='utf-8')
            start = time.time()
            #params = {
            #    "image":img_base64
            #}
            params = {
                "url":img_path
            }
            resp = await client.post(ocr_url, data=params, timeout=5,follow_redirects=True)
        end = time.time()
        logger.success(f"OCR结果：{resp.text},耗时{end-start:.4f}s\n图片url:{img_path}")
        return resp.text
    except:
        logger.error(traceback.format_exc())
        return ''
    
async def upload_OcrResult(result_text,filename):
    try:
        params = {
            "md5": filename,
            "text": b64encode(result_text.encode('utf-8')).decode('utf-8')
        }
        async with httpx.AsyncClient(headers=headers) as client:
            resp = await client.post(upload_url,json=params)
            result= resp.json()
        if result['code'] == 200:
            await downlod_OcrResult()
    except:
        logger.error(traceback.format_exc())
        
async def downlod_OcrResult():
    try:
        async with httpx.AsyncClient(headers=headers,timeout=None) as client:
            resp = await client.get(download_url)
            result = resp.json()
            with open(ocr_data_path, 'w', encoding='UTF-8') as f:
                if result['code'] == 200 and result['data']:
                    json.dump(result['data'], f)
                    global ocr_filename_data
                    ocr_filename_data = result['data']
                else:
                    ocr_filename_data = json.load(open(ocr_data_path, 'r', encoding='utf8'))
        return
    except:
        ocr_filename_data = json.load(open(ocr_data_path, 'r', encoding='utf8'))
        logger.error(traceback.format_exc())


import httpx
import traceback
from pathlib import Path
from nonebot.log import logger
from httpx import ConnectTimeout
from asyncio.exceptions import TimeoutError

dir_path = Path(__file__).parent

  
async def get_pupu_msg():
    try:
        url = 'https://v1.hitokoto.cn'
        params = {
    }
        async with httpx.AsyncClient() as client:
            resp = await client.get(url, params=params, timeout=5)
            result = resp.json()
        if resp.status_code == 200:
            return result['hitokoto']
        else:
            return '噗噗出问题了>_<'
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
    except Exception:
        logger.error(traceback.format_exc())
        return '噗噗出问题了>_<'
    


from typing import List
import httpx
import traceback
import jinja2
import re
import time
from pathlib import Path
from ..data_source import servers,set_infoparams,set_damageColor,set_winColor,set_upinfo_color
from ..utils import match_keywords
from ..publicAPI import get_AccountIdByName,check_yuyuko_cache
from nonebot_plugin_htmlrender import html_to_pic
from nonebot import get_driver
from nonebot.log import logger
from httpx import ConnectTimeout
from asyncio.exceptions import TimeoutError

dir_path = Path(__file__).parent.parent
template_path = dir_path / "template"
env = jinja2.Environment(
    loader=jinja2.FileSystemLoader(template_path), enable_async=True
)
env.globals.update(set_damageColor=set_damageColor,set_winColor=set_winColor,set_upinfo_color=set_upinfo_color,time=time,int=int,abs=abs,enumerate=enumerate)

headers = {
    'Authorization': get_driver().config.api_token
}
  
async def get_sx_info(server_type,info,bot,ev):
    try:
        url,params = '',''
        if isinstance(info,List):
            if len(info) == 0:
                params = {
                    "server": "QQ",
                    "accountId": int(ev.user_id)
                    }
            for i in info:
                if str(i).lower() == 'me':
                    params = {
                    "server": server_type,
                    "accountId": int(ev.user_id)
                    }
                    break
                match = re.search(r"CQ:at,qq=(\d+)",i)
                if match:
                    params = {
                    "server": server_type,
                    "accountId": int(match.group(1))
                    }
                    break
            if not params and len(info) == 2:
                param_server,info = await match_keywords(info,servers)
                if param_server:
                    param_accountid = await get_AccountIdByName(param_server,str(info[0]))
                    if isinstance(param_accountid,int):
                        params = {
                        "server": param_server,
                        "accountId": param_accountid
                        }
                    else:
                        return f"{param_accountid}"
                else:
                    return '服务器参数似乎输错了呢'
            elif params:
                logger.success(f"{params}")
            else:
                return '您似乎准备用游戏昵称查询，请检查参数中是否包含服务器和游戏昵称，以空格区分，如果您准备查询单船战绩，请带上ship参数'
        else:
            return '参数似乎出了问题呢'
        url = 'https://api.wows.shinoaki.com/public/wows/christmas/ship/christmas'
        logger.success(f"下面是本次请求的参数，如果遇到了问题，请将这部分连同报错日志一起发送给麻麻哦\n{url}\n{params}")
        async with httpx.AsyncClient(headers=headers) as client:
            resp = await client.get(url, params=params, timeout=None)
            result = resp.json()
            logger.success(f"本次请求返回的状态码:{result['code']}")
            logger.success(f"本次请求服务器计算时间:{result['queryTime']}")
        if result['code'] == 200 and result['data']:
            template = env.get_template("wws-sx.html")
            template_data = await set_infoparams(result['data'])
            content = await template.render_async(template_data)
            return await html_to_pic(content, wait=0, viewport={"width": 920, "height": 1000})
        elif result['code'] == 403:
            return f"{result['message']}\n请先绑定账号"
        elif result['code'] == 500:
            return f"{result['message']}\n这是服务器问题，请联系雨季麻麻"
        else:
            return f"{result['message']}"
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return '请求超时了，请过会儿再尝试哦~'
    except Exception:
        logger.error(traceback.format_exc())
        return 'wuwuwu出了点问题，请联系麻麻解决'


from typing import List
import httpx
import traceback
import re
from pathlib import Path
from ..data_source import servers
from ..utils import match_keywords
from ..publicAPI import get_AccountIdByName,check_yuyuko_cache
from ..data_source import nations,shiptypes,levels
from nonebot.log import logger
from nonebot import get_driver
from httpx import ConnectTimeout
from asyncio.exceptions import TimeoutError

dir_path = Path(__file__).parent

headers = {
    'Authorization': get_driver().config.api_token
}
  
async def roll_ship(server_type,infolist,bot,ev):
    try:
        param_nation,infolist = await match_keywords(infolist,nations)
        if not param_nation:
            param_nation = ''
        
        param_shiptype,infolist = await match_keywords(infolist,shiptypes)
        if not param_shiptype:
            param_shiptype = ''
        
        param_level,infolist = await match_keywords(infolist,levels)
        if not param_level:
            param_level = ''
        params = {
            "accountId": ev.user_id,
            "server": server_type,
            "county":param_nation,
            "level":param_level,
            "shipName":'',
            "shipType":param_shiptype
        }
        url = 'https://api.wows.shinoaki.com/public/wows/roll/ship/roll'
        logger.success(f"下面是本次请求的参数，如果遇到了问题，请将这部分连同报错日志一起发送给麻麻哦\n{url}\n{params}")
        async with httpx.AsyncClient(headers=headers) as client:
            resp = await client.post(url, json=params, timeout=None)
            logger.success(f"本次请求返回的状态码:{resp.status_code}")
            result = resp.json()
        if result['code'] == 200 and result['data']:
            msg = f"本次roll到了{result['data']['shipNameCn']}"
        elif result['code'] == 403:
            return f"{result['message']}\n请先绑定账号"
        elif result['code'] == 500:
            return f"{result['message']}\n这是服务器问题，请联系雨季麻麻"
        else:
            return f"{result['message']}"
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
        return '请求超时了，请过会儿再尝试哦~'
    except Exception:
        logger.error(traceback.format_exc())
        return 'wuwuwu出了点问题，请联系麻麻解决'


from typing import List
import httpx
import traceback
import jinja2
import re
import time
import asyncio
from pathlib import Path
from os import getcwd
from .data_source import servers,set_damageColor,set_winColor,set_upinfo_color,select_prvalue_and_color
from .publicAPI import get_all_shipList
from nonebot_plugin_htmlrender import html_to_pic
from nonebot import get_driver
from nonebot.log import logger
from nonebot.adapters.onebot.v11 import MessageSegment
from httpx import ConnectTimeout
from asyncio.exceptions import TimeoutError
from .utils import match_keywords,get_bot
from threading import Thread
from playwright.async_api import async_playwright

dir_path = Path(__file__).parent
template_path = dir_path / "template"
env = jinja2.Environment(
    loader=jinja2.FileSystemLoader(template_path), enable_async=True
)
env.globals.update(set_damageColor=set_damageColor,set_winColor=set_winColor,set_upinfo_color=set_upinfo_color,time=time,int=int,abs=abs,enumerate=enumerate)

headers = {
    'Authorization': get_driver().config.api_token
}

#all_shipList = asyncio.run(get_all_shipList())

async def send_realTime_message(data):
    try:
        global all_shipList
        if not all_shipList:
            all_shipList = await get_all_shipList()
        bot = get_bot()
        template = env.get_template("wws-realTime.html")
        template_data = await set_realTime_params(data)
        content = await template.render_async(template_data)
        print(content)
        async with async_playwright() as p:
            browser = await p.chromium.launch()
            page = await browser.new_page()
            await page.goto(f"file://{getcwd()}")
            await page.set_content(content, wait_until="networkidle")
            img = await page.screenshot(full_page=True)
            await browser.close()
        await bot.send_group_msg(group_id=639178962,message=f"[测试功能]雨季刚刚进入了一场战斗\n{MessageSegment.image(img)}")
    except Exception:
        logger.error(traceback.format_exc())
        return
    
async def set_realTime_params(data):
    try:
        player_count = len(data['infoList'])
        for each_player in data['infoList']:
            each_player['shipImgSmall'] = None
            for each_ship in all_shipList:
                if each_ship['id'] == each_player['shipId']:
                    if each_ship['shipNameCn']:
                        each_player['shipName'] = each_ship['shipNameCn']
                    else:
                        each_player['shipName'] = each_ship['name']
                    each_player['shipImgSmall'] = each_ship['imgSmall']
                    each_player['shipIdValue'] = f"{each_ship['shipIdValue']}"
                    break
            if each_player['pvp']:
                each_player['pvp']['pr_name'],each_player['pvp']['pr_color'] = await select_prvalue_and_color(each_player['pvp']['pr'])
            if each_player['ship']:
                each_player['ship']['pr_name'],each_player['ship']['pr_color'] = await select_prvalue_and_color(each_player['ship']['pr'])
        data['player_count'] = player_count
        data['template_path'] = template_path
        logger.success(data)
        return data
    except Exception:
        logger.error(traceback.format_exc())
        return None
    
async def send_message(content):
    bot = get_bot()
    img = await html_to_pic(content, wait=0, viewport={"width": 1800, "height": 1000})
    await bot.send_group_msg(group_id=574432871,message=MessageSegment.image(img))